-- get all RTDOSE and corresponding RTSTRUCT from a server
-- and make a table allowing worldmatch processing of that data

require('persistence')

server = 'CONQUESTSRV1'
url = '[DICOM local server]:'

grotetabel = {}

f = io.open('patientURLgenerationLog.txt', 'wt')
                              
a=DicomObject:new()
a.QueryRetrieveLevel='PATIENT'
a.PatientID=''
listofpatients=dicomquery(server, 'PATIENT', a)
for i=0, #listofpatients-1 do
  a=DicomObject:new()
  a.QueryRetrieveLevel='IMAGE'
  a.PatientID=listofpatients[i].PatientID
  a.SOPInstanceUID=''
  a.Modality = 'RTDOSE'
  listofrtdose=dicomquery(server, 'IMAGE', a)
  
  --print out error in case of no dose file
  if #listofrtdose == 0 then
     f:write('i: ' .. i .. ' PatNr ' .. listofpatients[i].PatientID .. ' - no doses-ERROR\n' )
  else  
    f:write('i: ' .. i .. ' PatNr ' .. listofpatients[i].PatientID .. '\n')
    for j=0, #listofrtdose-1 do
      f:write('\tdose : ' .. j .. ' ')
      dosisinfo = {}
      completeDose = true; --Dose is always complete unless proven otherwise
      
      --read the dose
      dose = DicomObject:new()
      dose:Read(listofrtdose[j].PatientID .. ':' .. listofrtdose[j].SOPInstanceUID)
      
      --read the corresponding plan if there is a reference
      if not (dose.ReferencedRTPlanSequence==nil) then
        
        --in case there is a single beam, continue, we don't like this 
        if not(dose.ReferencedRTPlanSequence.ReferencedFractionGroupSequence==nil) then
          if not(dose.ReferencedRTPlanSequence.ReferencedFractionGroupSequence.ReferencedBeamSequence==nil) then
            f:write('ERROR: Single beam dose plan\n')
            completeDose = false; --single beam kick out
            
          else--Is there a reference to the plan?
            if not (dose.ReferencedRTPlanSequence.ReferencedSOPInstanceUID==nil) then
          plan = DicomObject:new()
          plan:Read(listofrtdose[j].PatientID .. ':' .. dose.ReferencedRTPlanSequence.ReferencedSOPInstanceUID)
          
          --does the plan exists in the database?
          if not plan.SOPInstanceUID then
            f:write("ERROR: no RTPLAN corresponding to the dose\n");
            completeDose = false;
          end      
                
          --read the corresponding struct if there is a reference
          if not (plan.ReferencedStructureSetSequence==nil) then
            if not (plan.ReferencedStructureSetSequence.ReferencedSOPInstanceUID==nil) then
              
              struct = DicomObject:new()
              struct:Read(listofrtdose[j].PatientID .. ':' .. plan.ReferencedStructureSetSequence.ReferencedSOPInstanceUID)
              --print(struct.StudyInstanceUID, struct.SeriesInstanceUID, struct.SOPInstanceUID)
              --does the struct exists in the database?
              if not struct.SOPInstanceUID then
                f:write("ERROR: no RTSTRUCT corresponding to the plan\n");
                completeDose = false;
              end
              
              --read the corresponding CT url if there is a reference to it
              if not (struct.ReferencedStudySequence==nil) then
                if not (struct.ReferencedStudySequence.ReferencedSOPInstanceUID==nil) then
                  --get the reference CT from the struct
                  ctstudy  = struct.ReferencedStudySequence.ReferencedSOPInstanceUID
                  
                  if not (struct.ReferencedFrameOfReferenceSequence==nil) then
                    if not (struct.ReferencedFrameOfReferenceSequence.RTReferencedStudySequence==nil) then
                      if not (struct.ReferencedFrameOfReferenceSequence.RTReferencedStudySequence.RTReferencedSeriesSequence==nil) then
                        if (not (struct.ReferencedFrameOfReferenceSequence.RTReferencedStudySequence.RTReferencedSeriesSequence.SeriesInstanceUID==nil)) and
                           (not  (struct.ReferencedFrameOfReferenceSequence.RTReferencedStudySequence.RTReferencedSeriesSequence.ContourImageSequence==nil)) then
                            ctseries = struct.ReferencedFrameOfReferenceSequence.RTReferencedStudySequence.RTReferencedSeriesSequence.SeriesInstanceUID
                            
                            if not(struct.ReferencedFrameOfReferenceSequence.RTReferencedStudySequence.RTReferencedSeriesSequence.ContourImageSequence.ReferencedSOPInstanceUID==nil) then
                              ctsop    = struct.ReferencedFrameOfReferenceSequence.RTReferencedStudySequence.RTReferencedSeriesSequence.ContourImageSequence.ReferencedSOPInstanceUID                    --print(ctstudy, ctseries, ctsop)
                            
                              --read the scan from the database
                              ct       = DicomObject:new()
                              ct:Read(listofrtdose[j].PatientID .. ':' .. ctsop)
                            else
                              f:write("ERROR: struct.ReferencedFrameOfReferenceSequence.RTReferencedStudySequence.RTReferencedSeriesSequence.ContourImageSequence.ReferencedSOPInstanceUID\n");
                              completeDose = false;
                          end
                        else
                          f:write("ERROR: struct.ReferencedFrameOfReferenceSequence ..SeriesInstanceUID or ..ContourImageSequence \n");
                          completeDose = false;
                        end
                      else
                        f:write("ERROR: struct.ReferencedFrameOfReferenceSequence.RTReferencedStudySequence.RTReferencedSeriesSequence\n");
                        completeDose = false;
                      end
                    else
                      f:write("ERROR: struct.ReferencedFrameOfReferenceSequence.RTReferencedStudySequence\n");
                      completeDose = false;
                    end
                  else
                    f:write("ERROR: struct.ReferencedFrameOfReferenceSequence\n");
                    completeDose = false;
                  end
                else
                  f:write("ERROR: struct.ReferencedStudySequence.ReferencedSOPInstanceUID\n");
                  completeDose = false;
                end
              else
                f:write("ERROR: No struct.ReferencedStudySequence\n");
                completeDose = false;
              end
            
              --was the CT in the DICOM server available?
              if not ct.SOPInstanceUID then
                f:write("ERROR: no CT scan corresponding to the dose\n");
                completeDose = false;
              end
            else
              f:write("ERROR: No plan.ReferencedStructureSetSequence.ReferencedSOPInstanceUID\n");
              completeDose = false;
            end
          else
            f:write("ERROR: No plan.ReferencedStructureSetSequence\n");
            completeDose = false;
          end
        else
          f:write("ERROR: No dose.ReferencedRTPlanSequence.ReferencedSOPInstanceUID\n");
          completeDose = false;
        end
      
          end
        else
          --Is there a reference to the plan?
          if not (dose.ReferencedRTPlanSequence.ReferencedSOPInstanceUID==nil) then
            plan = DicomObject:new()
            plan:Read(listofrtdose[j].PatientID .. ':' .. dose.ReferencedRTPlanSequence.ReferencedSOPInstanceUID)
            
            --does the plan exists in the database?
            if not plan.SOPInstanceUID then
              f:write("ERROR: no RTPLAN corresponding to the dose\n");
              completeDose = false;
            end      
                  
            --read the corresponding struct if there is a reference
            if not (plan.ReferencedStructureSetSequence==nil) then
              if not (plan.ReferencedStructureSetSequence.ReferencedSOPInstanceUID==nil) then
                
                struct = DicomObject:new()
                struct:Read(listofrtdose[j].PatientID .. ':' .. plan.ReferencedStructureSetSequence.ReferencedSOPInstanceUID)
                --print(struct.StudyInstanceUID, struct.SeriesInstanceUID, struct.SOPInstanceUID)
                --does the struct exists in the database?
                if not struct.SOPInstanceUID then
                  f:write("ERROR: no RTSTRUCT corresponding to the plan\n");
                  completeDose = false;
                end
                
                --read the corresponding CT url if there is a reference to it
                if not (struct.ReferencedStudySequence==nil) then
                  if not (struct.ReferencedStudySequence.ReferencedSOPInstanceUID==nil) then
                    --get the reference CT from the struct
                    ctstudy  = struct.ReferencedStudySequence.ReferencedSOPInstanceUID
                    
                    if not (struct.ReferencedFrameOfReferenceSequence==nil) then
                      if not (struct.ReferencedFrameOfReferenceSequence.RTReferencedStudySequence==nil) then
                        if not (struct.ReferencedFrameOfReferenceSequence.RTReferencedStudySequence.RTReferencedSeriesSequence==nil) then
                          if (not (struct.ReferencedFrameOfReferenceSequence.RTReferencedStudySequence.RTReferencedSeriesSequence.SeriesInstanceUID==nil)) and
                             (not  (struct.ReferencedFrameOfReferenceSequence.RTReferencedStudySequence.RTReferencedSeriesSequence.ContourImageSequence==nil)) then
                              ctseries = struct.ReferencedFrameOfReferenceSequence.RTReferencedStudySequence.RTReferencedSeriesSequence.SeriesInstanceUID
                              
                              if not(struct.ReferencedFrameOfReferenceSequence.RTReferencedStudySequence.RTReferencedSeriesSequence.ContourImageSequence.ReferencedSOPInstanceUID==nil) then
                                ctsop    = struct.ReferencedFrameOfReferenceSequence.RTReferencedStudySequence.RTReferencedSeriesSequence.ContourImageSequence.ReferencedSOPInstanceUID                    --print(ctstudy, ctseries, ctsop)
                              
                                --read the scan from the database
                                ct       = DicomObject:new()
                                ct:Read(listofrtdose[j].PatientID .. ':' .. ctsop)
                              else
                                f:write("ERROR: struct.ReferencedFrameOfReferenceSequence.RTReferencedStudySequence.RTReferencedSeriesSequence.ContourImageSequence.ReferencedSOPInstanceUID\n");
                                completeDose = false;
                            end
                          else
                            f:write("ERROR: struct.ReferencedFrameOfReferenceSequence ..SeriesInstanceUID or ..ContourImageSequence \n");
                            completeDose = false;
                          end
                        else
                          f:write("ERROR: struct.ReferencedFrameOfReferenceSequence.RTReferencedStudySequence.RTReferencedSeriesSequence\n");
                          completeDose = false;
                        end
                      else
                        f:write("ERROR: struct.ReferencedFrameOfReferenceSequence.RTReferencedStudySequence\n");
                        completeDose = false;
                      end
                    else
                      f:write("ERROR: struct.ReferencedFrameOfReferenceSequence\n");
                      completeDose = false;
                    end
                  else
                    f:write("ERROR: struct.ReferencedStudySequence.ReferencedSOPInstanceUID\n");
                    completeDose = false;
                  end
                else
                  f:write("ERROR: No struct.ReferencedStudySequence\n");
                  completeDose = false;
                end
              
                --was the CT in the DICOM server available?
                if not ct.SOPInstanceUID then
                  f:write("ERROR: no CT scan corresponding to the dose\n");
                  completeDose = false;
                end
              else
                f:write("ERROR: No plan.ReferencedStructureSetSequence.ReferencedSOPInstanceUID\n");
                completeDose = false;
              end
            else
              f:write("ERROR: No plan.ReferencedStructureSetSequence\n");
              completeDose = false;
            end
          else
            f:write("ERROR: No dose.ReferencedRTPlanSequence.ReferencedSOPInstanceUID\n");
            completeDose = false;
          end
        end
      else
        f:write("ERROR: No dose.ReferencedRTPlanSequence\n");
        completeDose = false;
      end
        
      --if all is there, you can add all information to the table dosisinfo      
      if completeDose then
        --store the data to identify this specific plan
        dosisinfo.PatientID = listofpatients[i].PatientID
        dosisinfo.PlanName = plan.RTPlanName
        dosisinfo.PlanDate = plan.RTPlanDate
        dosisinfo.ScanDate  = ct.StudyDate
        
        dosisinfo.CTUrl = url..listofpatients[i].PatientID..':'..listofpatients[i].PatientID..'.patient\\'..
          ct.StudyInstanceUID .. '.study\\' ..
          ct.SeriesInstanceUID .. '.series\\' ..
          'any.scan'
        
        dosisinfo.CTIdentifiers= 
          { PatientID = listofpatients[i].PatientID,
            StudyInstanceUID=ct.StudyInstanceUID,
            SeriesInstanceUID=ct.SeriesInstanceUID,
            Modality=ct.Modality
          }
        
        --in case of a multiframe scan, it needs to have a different extension 
        if not(dose.NumberOfFrames==nil) or (dose.NumberOfFrames > 1) then
          dosisinfo.DoseUrl = url..listofpatients[i].PatientID..':'..listofpatients[i].PatientID..'.patient\\'..
              dose.StudyInstanceUID .. '.study\\' ..
              dose.SeriesInstanceUID .. '.series\\' ..
              dose.SOPInstanceUID .. '&MfScan.dose'
          dosisinfo.DoseIdentifiers= 
            { PatientID = listofpatients[i].PatientID,
              StudyInstanceUID=dose.StudyInstanceUID,
              SeriesInstanceUID=dose.SeriesInstanceUID,
              SOPInstanceUID=dose.SOPInstanceUID,
              Modality=dose.Modality
            }
        else
          dosisinfo.DoseUrl = url..listofpatients[i].PatientID..':'..listofpatients[i].PatientID..'.patient\\'..
              dose.StudyInstanceUID .. '.study\\' ..
              dose.SeriesInstanceUID .. '.series\\' ..
              'any.dose'
          dosisinfo.DoseIdentifiers= 
            { PatientID = listofpatients[i].PatientID,
              StudyInstanceUID=dose.StudyInstanceUID,
              SeriesInstanceUID=dose.SeriesInstanceUID,
              Modality=dose.Modality
            }
        end
        
        dosisinfo.StructUrl = url..listofpatients[i].PatientID..':'..listofpatients[i].PatientID..'.patient\\'..
            ctstudy .. '.study\\' ..
            ctseries .. '.series\\' ..
            ctsop .. '.scan\\' ..
            struct.SOPInstanceUID .. '.structset'
        
        dosisinfo.StructIdentifiers= 
          { PatientID = listofpatients[i].PatientID,
            StudyInstanceUID=ct.StudyInstanceUID,
            SeriesInstanceUID=ct.SeriesInstanceUID,
            SOPInstanceUID=struct.SOPInstanceUID,
            Modality=struct.Modality
          }
        
        dosisinfo.structures = {}
        
        --print(#struct.StructureSetROISequence .. ' ' .. #struct.ROIContourSequence)
        if (not (struct.StructureSetROISequence==nil)) and (not (struct.ROIContourSequence==nil)) then
          for k=0, #struct.StructureSetROISequence-1 do          
            if not (struct.ROIContourSequence[k].ContourSequence==nil) then
              if tonumber(struct.ROIContourSequence[k].ContourSequence.NumberOfContourPoints) > 3 then 
                 -- table.insert(dosisinfo.structures, k,  struct.StructureSetROISequence[k].ROIName)
                 dosisinfo.structures[struct.StructureSetROISequence[k].ROINumber] = struct.StructureSetROISequence[k].ROIName
              end
            else
              f:write('ERROR: kicked out ' .. struct.StructureSetROISequence[k].ROIName .. '\n\t')
            end
          end
          --add the dosisinfo to the big table of all patients
          table.insert(grotetabel, dosisinfo)
          --print('CORRECT dose complete and inserted in table');
          f:write("dose correct\n")
          
        else
          f:write("ERROR: no struct.StructureSetROISequence or struct.ROIContourSequence\n");
          completeDose = false;
        end
      end
    end
    end
  persistence.store('allPatients.lua', grotetabel)
  f:flush();
end

f:close()

print(1);